
/*
 * KITNARF's Java Library <http://www.kitnarf.cz/java/>.
 * COPYRIGHT (C) 2009 Petr Frantík (www.kitnarf.cz). ALL RIGHTS RESERVED. 
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


package cz.kitnarf.colorfunction;

import java.awt.*;
import java.io.*;
import java.util.*;

import cz.kitnarf.packageexplorer.*;


//**************************************************************************************************************************
/**
 *Abstract class for calculation of colors from a color function.
 *
 *@author Petr Frantík
 *@version 20090404
 */
public abstract class ColorFunction
{



	//*****************************************************************************************
	/**
	 *Saves color distribution of this color function to specified file.
	 *
	 *@param path path to file. 
	 */
	public void saveDistribution(String path,int samples) throws IOException
	{
		PrintWriter pw=new PrintWriter(path+this.getClass().getSimpleName()+".txt");
		//
		Color bufColor=getColor(0);
		int count=1;
		int index=0;
		long startTime=System.currentTimeMillis();
		//
		//hledani distribuce
		System.out.println("Calculating distribution...");
		for(int i=1;i<samples;i++)
		{
			double c=1.0/(samples-1)*i;
			//System.out.println(c);
			Color color=getColor(c);
			//
			if(color.equals(bufColor)) count++;
			else
			{
				save(pw,index,count,bufColor);
				//
				index++;
				bufColor=color;
				count=1;
			}
		}
		save(pw,index,count,bufColor);
		//
		//
		long endTime=System.currentTimeMillis();
		System.out.println("Finished in: "+(endTime-startTime)+" ms");
		//
		pw.close();
	}



	//*****************************************************************************************
	/**
	 *Saves attributes of given color.
	 *
	 *@param pw a PrintWriter.
	 *@param index index of given color in this color function.
	 *@param count frequency of color in this color function.
	 */
	public static void save(PrintWriter pw,int index,int count,Color color)
	{
		pw.print(index+"	");
		pw.print(count+"	");
		pw.print(color.getRed()+"	");
		pw.print(color.getGreen()+"	");
		pw.print(color.getBlue());
		pw.println();
	}


	//*****************************************************************************************
	// SETRY, GETRY
	//*****************************************************************************************
	/**
	 *Calculates color from this color function. Coordinate outside
	 *of interval <0,1> will be corrected to the closest interval
	 *boundary.
	 *
	 *@param cd coordinate.
	 *@return color from this color function according to given coordinate.
	 */
	public Color getSafeColor(double cd)
	{
		//correction
		if(cd<0) cd=0;
		else if(cd>1.0) cd=1.0;
		//
		return getColor(cd);
	}



	//*****************************************************************************************
	/**
	 *Calculates color from this color function.
	 *@param cd coordinate.
	 *@return color from this color function according to given coordinate.
	 */
	public abstract Color getColor(double cd);



	//*****************************************************************************************
	// MAIN
	//*****************************************************************************************
	/** Main method. */
	public static void main(String[] arguments) throws Exception
	{
		//Loading of package content
		Repository repository=new Repository();
		repository.setAvailableColorFunctions();
		//
		//test branching
		if(arguments.length>0)
		{
			String cfName=arguments[0];
			//
			ColorFunction cf=repository.getColorFunctionInstance(cfName);
			//
			cf.saveDistribution("",1000000);
			new ScaleFrame(cf);
		}
		else
		{
			System.out.println("You can use following parameters:");
			for(String s:repository.getAvailableColorFunctionNames()) System.out.println(s);
		}
	}


}
