
/*
 * KITNARF's Java Library <http://www.kitnarf.cz/java/>.
 * COPYRIGHT (C) 2009 Petr Frantík (www.kitnarf.cz). ALL RIGHTS RESERVED. 
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


package cz.kitnarf.colorfunction;

import java.awt.*;
import java.io.*;
import java.util.*;

import cz.kitnarf.packageexplorer.*;


//**************************************************************************************************************************
/**
 *Stores and provides list of clases extended from ColorFunction which were
 *detected from the package class structure or the JAR file.
 *
 *@author Petr Frantík
 *@version 20090404
 */
public class Repository
{
	private HashMap<String,Class> classMap;



	//*****************************************************************************************
	/**
	 *Create new color function.
	 *@param type type of a color function.
	 *@return new instance of the color function.
	 */
	@Deprecated
	public static ColorFunction createColorFunction(int type) throws IllegalArgumentException
	{
		if(type==0) return new GrayColorFunction();
		else if(type==1) return new BilinearColorFunction();
		else if(type==2) return new TrilinearColorFunction();
		else if(type==3) return new TetralinearColorFunction();
		else if(type==4) return new SextilinearColorFunction();
		else if(type==5) return new SeptilinearColorFunction();
		else if(type==6) return new BicircularColorFunction();
		else if(type==7) return new EllipticColorFunction();
		else if(type==8) return new SuperellipticColorFunction();
		else if(type==9) return new SpiralColorFunction();
		else if(type==10) return new FullSurfaceColorFunction();
		else throw new IllegalArgumentException();
	}



	//*****************************************************************************************
	// SETRY, GETRY
	//*****************************************************************************************
	/**
	 *Finds and sets map of available color functions.
	 */
	public void setAvailableColorFunctions() throws ClassNotFoundException
	{
		java.util.List<Class> classes=findAvailableColorFunctions();
		classMap=new HashMap<String,Class>(classes.size());
		//
		for(Class c:classes)
		{
			classMap.put(c.getSimpleName(),c);
		}
	}




	//*****************************************************************************************
	/**
	 *Finds and gets list of available color functions.
	 *@return list of available color functions.
	 */
	public static java.util.List<Class> findAvailableColorFunctions() throws ClassNotFoundException
	{
		java.util.List<Class> allClasses=PackageExplorer.getClasses("cz.kitnarf.colorfunction");
		java.util.ArrayList<Class> subClasses=new java.util.ArrayList<Class>();
		//
		//odfiltrovani jinych nez potomku ColorFunction
		for(Class c:allClasses)
		{
			if(c.getSuperclass().equals(ColorFunction.class)) subClasses.add(c);
		}
		//
		return subClasses;
	}




	//*****************************************************************************************
	/**
	 *Gets array of available color function names.
	 *@param classes list of available classes.
	 *@return array of the available color function simple names.
	 */
	public static String[] getAvailableColorFunctionNames(java.util.List<Class> classes) throws ClassNotFoundException
	{
		String[] names=new String[classes.size()];
		//
		//ziskani simple names
		int i=0;
		for(Class c:classes)
		{
			names[i]=c.getSimpleName();
			i++;
		}
		//
		return names;
	}


	//*****************************************************************************************
	/**
	 *Gets set of available color function names from repository.
	 *@return set of available color function names.
	 */
	public Set<String> getAvailableColorFunctionNames()
	{
		return classMap.keySet();
	}


	//*****************************************************************************************
	/**
	 *Gets instance of the color function.
	 *@param name name of the color function.
	 *@return instance of the color function.
	 */
	public ColorFunction getColorFunctionInstance(String name) throws ClassNotFoundException,InstantiationException,IllegalAccessException
	{
		Class c=classMap.get(name);
		if(c==null) throw new ClassNotFoundException("Class named "+name+" not found in repository.");
		//
		Object o=c.newInstance();
		if(o instanceof ColorFunction) return (ColorFunction) o;
		else throw new ClassNotFoundException("Class named "+name+" is not instance of ColorFunction.");
	}





}
