
/*
 * KITNARF's Java Library <http://www.kitnarf.cz/java/>.
 * COPYRIGHT (C) 2009 Petr Frantík (www.kitnarf.cz). ALL RIGHTS RESERVED. 
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


package cz.kitnarf.colorfunction;

import java.awt.*;


//**************************************************************************************************************************
/**
 *Sextilinear color function. This function is composed from six straight
 *segments. It has 1531 colors.
 *
 *@author Petr Frantík
 *@version 20090404
 */
public class SextilinearColorFunction extends ColorFunction
{



	//*****************************************************************************************
	/** {@inheritDoc} */
	@Override
	public Color getColor(double cd)
	{
		int[] ci=new int[3];
		//
		//
		if(cd<=0.166666666666)										//Y-M
		{
			ci[0]=255;
			ci[1]=(int)((0.166666666666-cd)*1531+0.833333333333);
			ci[2]=255-ci[1];
		}
		else if(cd<=0.333333333333)								//M-R
		{
			ci[0]=255;
			ci[1]=0;
			ci[2]=(int)((0.333333333333-cd)*1531+0.666666666666);
		}
		else if(cd<=0.5)													//R-0
		{
			ci[0]=(int)((0.5-cd)*1531+0.5);
			ci[1]=0;
			ci[2]=0;
		}
		else if(cd<=0.666666666666)								//0-B
		{
			ci[0]=0;
			ci[1]=0;
			ci[2]=(int)((cd-0.5)*1531+0.5);
		}
		else if(cd<=0.833333333333)								//B-G
		{
			ci[0]=0;
			ci[1]=(int)((cd-0.666666666666)*1531+0.666666666666);
			ci[2]=255-ci[1];
		}
		else																			//G-C
		{
			//
			ci[0]=0;
			ci[1]=255;
			ci[2]=(int)((cd-0.833333333333)*1531+0.833333332);
			//
			if(ci[0]>255)	System.out.println(ci[0]+""+((cd-0.833333333333)*1531));
		}
		//
		return new Color(ci[0],ci[1],ci[2]);
	}



	//*****************************************************************************************
	/**
	 *Calculate alternative color from color scale
	 *param cd coordinate
	 *return color from color scale
	 */
	public Color getAlternativeColor(double cd)
	{
		int[] ci=new int[3];
		//
		//
		if(cd<=0.166666666666)										//1-M
		{
			ci[0]=255;
			ci[1]=(int)((0.166666666666-cd)*1531+0.833333333333);
			ci[2]=255;
		}
		else if(cd<=0.333333333333)								//M-R
		{
			ci[0]=255;
			ci[1]=0;
			ci[2]=(int)((0.333333333333-cd)*1531+0.666666666666);
		}
		else if(cd<=0.5)													//R-0
		{
			ci[0]=(int)((0.5-cd)*1531+0.5);
			ci[1]=0;
			ci[2]=0;
		}
		else if(cd<=0.666666666666)								//0-B
		{
			ci[0]=0;
			ci[1]=0;
			ci[2]=(int)((cd-0.5)*1531+0.5);
		}
		else if(cd<=0.833333333333)								//B-G
		{
			ci[0]=0;
			ci[1]=(int)((cd-0.666666666666)*1531+0.666666666666);
			ci[2]=255-ci[1];
		}
		else																			//G-Y
		{
			//
			ci[0]=(int)((cd-0.833333333333)*1531+0.833333332);
			ci[1]=255;
			ci[2]=0;
			//
			if(ci[0]>255)	System.out.println(ci[0]+""+((cd-0.833333333333)*1531));
		}
		//
		return new Color(ci[0],ci[1],ci[2]);
	}

}
